      SUBROUTINE FFTN(LG,G,IS,L,WK1,WK2)
C FFTN finds the discrete Fourier transform of the sequence G of length LG
C for input IS=-1. For input IS=1, it finds the inverse of the DFT
C sequence G of length LG. LG can be any positive integer. The subroutine
C uses the method of Bluestein in which the transform is converted to
C a convolution. The sequences entering the convolution are padded with
C zeroes to length L, an integral power of two, equal to or larger than 3LG-2.
C The convolution is then performed by finding the DFTs of each sequence
C using the subroutine FFT2N, multiplying them, and then finding the inverse
C of the resulting DFT. WK1 and WK2 are work area vectors, defined in
C the main programme, of length L or larger. On output, the transformed
C sequence replaces the input sequence.
C  
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DOUBLE PRECISION PI,ARG,ARGJ2,FACT,DC,DS
      DIMENSION G(LG),WK1(L),WK2(L)
C Define pi.
      PI=3.14159265358979D0
C Define common factor in argument of W.
      ARG=PI*DFLOAT(IS)/DFLOAT(LG)
C Determine if DFT or inverse DFT.
      FACT=1.D0
      IF(IS.EQ.1) FACT=FACT/DFLOAT(LG)
C Find N, the power of two equal to, or next larger than 3*LG-2,
C the length of the convolution.
      LGT3M2=3*LG-2
      N=0
      I=1
 10   IF(I.GE.LGT3M2) GO TO 20
      I=I*2
      N=N+1
      GO TO 10
 20   CONTINUE
      L=2**N
C Zero fill padded sequences being convolved.
      DO 30 I=1,L
        WK1(I)=(0.D0,0.D0)
        WK2(I)=(0.D0,0.D0)
 30   CONTINUE
C Find sequences to be convolved.
      DO 40 J=1,LG
        JM1=J-1
        JM1S=JM1*JM1
        ARGJ2=ARG*DFLOAT(JM1S)
        DC=DCOS(ARGJ2)
        DS=DSIN(ARGJ2)
        WK1(J)=FACT*G(J)*DCMPLX(DC,DS)
C Store weights for final DFT calculation.
        G(J)=DCMPLX(DC,DS)
        JPLGM1=J+LG-1
        WK2(JPLGM1)=DCMPLX(DC,-DS)
        IF(J.EQ.1) GO TO 40
        LGMJP1=LG-J+1
        WK2(LGMJP1)=WK2(JPLGM1)
 40   CONTINUE
C Find FFTs of sequences entering convolution.
      IS=-1
      CALL FFT2N(L,WK1,IS)
      CALL FFT2N(L,WK2,IS)
C Multiply FFTs to find FFT of convolution.
      DO 50 I=1,L
        WK1(I)=WK1(I)*WK2(I)
 50   CONTINUE
C Find convolution by inverse DFT.
      IS=1
      CALL FFT2N(L,WK1,IS)
C Weight convolution to obtain DFT.
      DO 60 I=1,LG
        IPLGM1=I+LG-1
        G(I)=G(I)*WK1(IPLGM1)
 60   CONTINUE
      RETURN   
      END

