      SUBROUTINE HHOLDER(V,U,N,M,MMAX)
C
C The subroutine HHOLDER annihilates the components of the vector V(M)
C beyond the Nth component for N<M. The Householder vector U is calculated
C and the annihilated vector W replaces V on return.
C
      DOUBLE COMPLEX U(MMAX),V(MMAX),W(MMAX)
      DOUBLE PRECISION ETA,AMAG,VWMAG,VR,VI
C No modification for N=M.
      IF(N.EQ.M) GO TO 10
C Find value of ETA.
      ETA=0.D0
      DO 11 I=N,M
        ETA=ETA+DREAL(V(I)*DCONJG(V(I)))
 11   CONTINUE
      ETA=DSQRT(ETA)
C Find magnitude and real and imaginary parts of V(N).
      AMAG=CDABS(V(N))
      VR=DREAL(V(N))
      VI=DIMAG(V(N))
C Find magnitude of V-W.
      VWMAG=2.D0*ETA*(ETA+AMAG)
      VWMAG=DSQRT(VWMAG)
C Construct vector U.
C Set initial values of components of U to zero.
      DO 12 I=1,M
        U(I)=(0.D0,0.D0)
 12   CONTINUE
C Set value of U(N).
      U(N)=V(N)+ETA*DCMPLX(VR/AMAG,VI/AMAG)
C Set values of remaining components of U.
      NP1=N+1
      DO 13 I=NP1,M
        U(I)=V(I)
 13   CONTINUE
C Normalize U to unit vector.
      DO 14 I=1,M
        U(I)=U(I)/VWMAG
 14   CONTINUE
C Initialize vector W as vector V.
      DO 15 I=1,M
        W(I)=V(I)
 15   CONTINUE  
C Construct vector W.
C Set value of W(N).
      W(N)=W(N)-VWMAG*U(N)
C Zero remaining components of W.
      DO 16 I=NP1,M
        W(I)=(0.D0,0.D0)
 16   CONTINUE
C Replace V by W.
      DO 17 I=1,M
        V(I)=W(I)
 17   CONTINUE  
 10   CONTINUE
      RETURN
      END
