      SUBROUTINE LEVSOL(LR,R,G,F,A)
C
C LEVSOL solves a linear system with a Toeplitz coefficient matrix by
C the Levinson recursive method. LR is the dimension of the system and R is
C the LR-length vector of autocorrelations for zero and positive lags,
C or in the general case, the first column vector of the coefficent matrix.
C G is the right-hand side vector, F is the solution vector and A is
c a vector representing the solution of an auxillary prediction error
C system used in the recursion. Since A is overwritten in the recursion,
C its previous values are stored in the vector A0.
C
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DIMENSION R(LR),G(LR),F(LR),A(LR),AO(LR)
C Solve 1x1 system
      F(1)=G(1)/R(1)
      IF(LR.EQ.1) RETURN
C Set first element of the auxillary vector to unity.
      A(1)=(1.D0,0.D0)
C Initialize alpha, beta and gamma.
      ALPHA=R(1)
      BETA=R(2)
      GAMMA=F(1)*R(2)       
C Begin loop over remaining equation systems.
      DO 10 L=2,LR
C Find k for L-1.
        AK=-BETA/DCONJG(ALPHA)
C Find last element of new auxillary vector.
        LM1=L-1
        A(L)=AK
C 2x2 system is a special case.
        IF(L.EQ.2) GO TO 20
C Store old auxillary vector elements.
        DO 40 I=2,LM1
          AO(I)=A(I)
 40     CONTINUE
C Find remaining elements of auxillary vector.
        LM2=L-2
        DO 50 J=1,LM2
          JP1=J+1
          K=L-J
          A(JP1)=A(JP1)+AK*DCONJG(AO(K))
 50     CONTINUE
C Update alpha.
 20     ALPHA=ALPHA+AK*DCONJG(BETA)
C Find q for L-1.
        Q=(G(L)-GAMMA)/DCONJG(ALPHA)
C Find last element of solution vector.
        F(L)=Q
C Find remaining elements of the solution vector.
        DO 60 J=1,LM1
          K=L-J+1
          F(J)=F(J)+Q*DCONJG(A(K))
 60     CONTINUE
C Return if solution is complete.
        IF(L.EQ.LR) RETURN
C Find new values of beta and gamma.
        BETA=(0.D0,0.D0)
        GAMMA=(0.D0,0.D0)
        DO 10 I=1,L
          K=L-I+2
          BETA=BETA+A(I)*R(K)
          GAMMA=GAMMA+F(I)*R(K)
 10   CONTINUE
      RETURN
      END
