      SUBROUTINE YPRIME(R,Y,A,YP,N1,C,RM,RHOM,MUM,LAMBDAM,GZEROM,N,
     1 PI,G,IR,RMIN,ANGS,COR,WES)
C This subroutine finds the derivatives, YP(6,6), of the six
C fundamental solutions represented by the propagator matrix, Y(6,6),
C in the mantle and crust (shell), at an arbitrary radius R
C for degree N>0. For N=0, the third and fourth solution vectors
C can be ignored. In the inner core, it finds the derivatives of
C the fundamental solutions in terms of the z-variables, which are
C transformations of the y-variables. 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION RM(100),RHOM(100),GZEROM(100),C(100,100),
     1 Y(6,6),YP(6,6),A(6,6)
      DOUBLE PRECISION MUM(100),LAMBDAM(100),MU,LAMBDA 
C Zero fill derivative matrix YP(I,J) and coefficient matrix A(I,J).
      DO 10 I=1,6
        DO 11 J=1,6
          YP(I,J)=0.D0
          A(I,J)=0.D0
 11     CONTINUE
 10   CONTINUE
C Set maximum number of points for interpolation.
      M1=100
C If R is less than or equal to RMIN, take properties to be those
C at the geocentre.
      IF(R.LE.RMIN) GO TO 12
C Interpolate Earth properties at radius R.
      CALL INTPL(R,RHO,N1,C,RM,RHOM,M1)
      CALL INTPL(R,MU,N1,C,RM,MUM,M1)
      CALL INTPL(R,LAMBDA,N1,C,RM,LAMBDAM,M1)         
      CALL INTPL(R,GZERO,N1,C,RM,GZEROM,M1)
      GO TO 13
C Assign geocentric Earth properties.
 12   RHO=RHOM(1)
      MU=MUM(1)
      LAMBDA=LAMBDAM(1)
      GZERO=(4.D0*PI*G*RHO-2.D0*WES)*R/3.D0
C Construct matrix A.
 13   AN=DFLOAT(N)
C Define factors.
      F1=AN*(AN+1.D0)
      F2=LAMBDA+2.D0*MU
      F3=3.D0*LAMBDA+2.D0*MU
      F4=RHO*GZERO
      F5=4.D0*PI*G*RHO
C Assign values to matrix elements.
      A(1,1)=-2.D0*LAMBDA/(F2*R)
      A(1,2)=1.D0/F2
      A(1,3)=F1*LAMBDA/(F2*R)
      A(1,4)=0.D0
      A(1,5)=0.D0
      A(1,6)=0.D0
      A(2,1)=4.D0*(-F4+MU*F3/(F2*R))/R-2.D0*RHO*WES-ANGS*RHO
      A(2,2)=-4.D0*MU/(F2*R)
      A(2,3)=F1*(F4-2.D0*MU*F3/(F2*R))/R+COR*RHO
      A(2,4)=F1/R
      A(2,5)=0.D0
      A(2,6)=-RHO
      IF(N.EQ.0.AND.IR.EQ.1) GO TO 14
      A(3,1)=-1.D0/R
      A(3,2)=0.D0
      A(3,3)=1.D0/R
      A(3,4)=1.D0/MU
      A(3,5)=0.D0
      A(3,6)=0.D0
      A(4,1)=(F4-2.D0*MU*F3/(F2*R))/R
C If degree is not zero, add Coriolis term.
      IF(N.NE.0) A(4,1)=A(4,1)+COR*RHO/F1
      A(4,2)=-LAMBDA/(F2*R)
      A(4,3)=2.D0*MU*((2.D0*F1-1.D0)*LAMBDA
     1 +(2.D0*F1-2.D0)*MU)/(F2*R*R) -ANGS*RHO
C If degree is not zero, add Coriolis term.
      IF(N.NE.0) A(4,3)=A(4,3)+COR*RHO/F1
      A(4,4)=-3.D0/R
      A(4,5)=-RHO/R
      A(4,6)=0.D0
 14   CONTINUE
      A(5,1)=F5
      A(5,2)=0.D0
      A(5,3)=0.D0
      A(5,4)=0.D0
      A(5,5)=0.D0
      A(5,6)=1.D0
      IF (N.EQ.0.AND.IR.EQ.1) GO TO 15
      A(6,1)=0.D0
      A(6,2)=0.D0
      A(6,3)=-F5*F1/R
      A(6,4)=0.D0
      A(6,5)=F1/(R*R)
 15   CONTINUE
      A(6,6)=-2.D0/R
C Branch to inner core calculation
      IF(IR.EQ.1) GO TO 16
C Multiply A into Y to find derivative matrix for shell.
      DO 17 I=1,6
        DO 18 J=1,6
          DO 19 K=1,6
            YP(I,J)=YP(I,J)+A(I,K)*Y(K,J)
 19       CONTINUE
 18     CONTINUE    
 17   CONTINUE
      GO TO 20
C Modify matrix A for inner core calculation.
 16   A(1,2)=A(1,2)/R
      A(2,1)=A(2,1)*R
      A(2,3)=A(2,3)*R
      A(2,6)=A(2,6)*R
      A(3,4)=A(3,4)/R
      A(4,1)=A(4,1)*R
      A(4,3)=A(4,3)*R
      A(4,5)=A(4,5)*R*R
      A(5,1)=A(5,1)/R
      A(5,6)=A(5,6)/R
      A(6,5)=A(6,5)*R
C Multiply A into Y to find YP for inner core.
      DO 21 I=1,6
        DO 22 J=1,3
          DO 23 K=1,6
            YP(I,J)=YP(I,J)+A(I,K)*Y(K,J)
 23       CONTINUE
 22     CONTINUE    
 21   CONTINUE
C Correct YP to give derivatives of z-variables.
C Set value of alpha plus one for first two solutions.
      ANM1=AN-1.D0
      DO 24 J=1,3
C Set value of alpha plus one for third solution
        IF(J.EQ.3) ANM1=AN+1.D0
        YP(1,J)=YP(1,J)-ANM1*Y(1,J)/R
        YP(2,J)=YP(2,J)-(ANM1-1.D0)*Y(2,J)/R
        YP(3,J)=YP(3,J)-ANM1*Y(3,J)/R
        YP(4,J)=YP(4,J)-(ANM1-1.D0)*Y(4,J)/R
        YP(5,J)=YP(5,J)-(ANM1+1.D0)*Y(5,J)/R
        YP(6,J)=YP(6,J)-ANM1*Y(6,J)/R
 24   CONTINUE
 20   CONTINUE
      RETURN
      END
