C                      PROGRAMME DECOMP.FOR
C
C DECOMP.FOR calculates the scalar decompression factor f for
C a particular Earth model. It first reads in the Earth model,
C converts it to SI units, and uses the subroutines SPMAT and INTPL
C to interpolate outer core values, using spline interpolation.
C The required quadrature for the calculation of the decompression
C factor as a function of radius is performed using Simpson's rule.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION R(100),RHO(100),GZERO(100),RI(300),RHOI(300),F(100),
     1 GZEROI(300),ENAME(10),NM(4),NI(4),NK(4),B(98,198),C(100,100)
      DOUBLE PRECISION MU(100),LAMBDA(100),MUI(300),LAMBDAI(300),
     1 LAMBDAM
      CHARACTER*20 EMODEL
C Read in Earth model file name.
      WRITE(6,10)
 10   FORMAT(1X,'Type in Earth model file name.')
      READ(5,11)EMODEL
 11   FORMAT(A20)
C Open Earth model file. 
      OPEN(UNIT=1,FILE=EMODEL,STATUS='OLD')
      OPEN(UNIT=2,FILE='decomp.dat',STATUS='UNKNOWN') 
C Set maximum dimensions for interpolation.
      M1=100
      M2=198
      M3=98
C Read in and write out Earth model.
C Read in and write out headers.
      READ(1,12)(ENAME(I),I=1,10)
 12   FORMAT(10A8)
      WRITE(6,13)(ENAME(I),I=1,10)
 13   FORMAT(11X,10A8)
      WRITE(2,12)(ENAME(I),I=1,10)
      READ(1,14)NN
 14   FORMAT(I10)
      READ(1,15)(NM(I),NI(I),I=1,4)
 15   FORMAT(8I10)
      WRITE(6,16)
 16   FORMAT(/16X,'Number of model points and',1X,
     1 'number of integration steps.'/)
      WRITE(6,15)(NM(I),NI(I),I=1,4)
      WRITE(6,17)
 17   FORMAT(//5X,'Radius',6X,'Rho',4X,'Lambda',7X,'Mu',6X,'Gzero')
      WRITE(6,18)
 18   FORMAT(6X,'(km)',5X,'(gm/cc)',2X,'(kbars)',4X,'(kbars)',2X,
     1'(cm/sec)'/)
C Read in Earth model, inner core, outer core, mantle and crust.
      K=0
      DO 19 M=1,4
        N1=NM(M)
        READ(1,20)(R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I),I=1,N1)
 20     FORMAT(1X,F10.1,F10.2,F10.1,F10.1,F10.1)
        DO 21 I=1,N1
C Scale Earth model to SI values and store.
          J=K+I
          RI(J)=R(I)*1.D3
          RHOI(J)=RHO(I)*1.D3
          LAMBDAI(J)=LAMBDA(I)*1.D8
          MUI(J)=MU(I)*1.D8
          GZEROI(J)=GZERO(I)*1.D-2
C Write  Earth model out to screen.
          WRITE(6,20)R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I) 
 21     CONTINUE 
        NK(M)=K
        K=K+N1
 19   CONTINUE
C Begin calculation of decompression factor f.
C Set up interpolation for the outer core.
      N1=NM(2)
      N2=2*N1-2
      N3=N1-2
C Put outer core values in active locations.
      DO 22 I=1,N1
        J=NK(2)+I
        R(I)=RI(J)
        RHO(I)=RHOI(J)
        MU(I)=MUI(J)
        LAMBDA(I)=LAMBDAI(J)
        GZERO(I)=GZEROI(J)
 22   CONTINUE
C Construct interpolation matrix.
      CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Begin integration by Simpson's rule.
      WRITE(6,23)
 23   FORMAT(//9X,'radius',9X,'exponent',7X,'f factor')
      EINT=0.D0
      F(N1)=1.D0 
      RKM=R(N1)*1.D-3
      WRITE(2,14)N1
      WRITE(2,24)R(N1),F(N1),RHO(N1)
 24   FORMAT(3D23.15) 
      WRITE(6,25)RKM,EINT,F(N1)
 25   FORMAT(/1X,F15.1,2F15.3)
      N1M1=N1-1
      DO 26 I=1,N1M1
C Interpolate rho, g, lambda to midpoint of interval.
        J=N1-I
        JP1=J+1
C Find stepsize for integration.
        H=0.5D0*(R(JP1)-R(J))
C Find midpoint of interval.
        RM=R(J)+H      
        CALL INTPL(RM,RHOM,N1,C,R,RHO,M1)
        CALL INTPL(RM,GMEAN,N1,C,R,GZERO,M1)
        CALL INTPL(RM,LAMBDAM,N1,C,R,LAMBDA,M1)
C Apply Simpson's rule.
        F1=RHO(J)*GZERO(J)/LAMBDA(J)
        F2=RHOM*GMEAN/LAMBDAM
        F3=RHO(JP1)*GZERO(JP1)/LAMBDA(JP1)
        EINT=EINT+H*(F1+4.D0*F2+F3)/3.D0
        F(J)=DEXP(EINT)
        RKM=R(J)*1.D-3
C Write out results.
        WRITE(2,24)R(J),F(J),RHO(J)
        WRITE(6,25)RKM,EINT,F(J)
 26   CONTINUE
C Interpolate decompression factor onto integration points.
C Find number of integration steps and interval size.
      M=NI(2)
      AM=DFLOAT(M)
      H=(R(N1)-R(1))/AM
      RKM=R(1)*1.D-3
      EINT=DLOG(F(1))
      FI=F(1)
      WRITE(2,25)RKM,EINT,FI
      DO 27 I=1,M
        AI=DFLOAT(I)
        XV=R(1)+AI*H
        CALL INTPL(XV,FI,N1,C,R,F,M1)
        EINT=DLOG(FI)
        RKM=XV*1.D-3
        WRITE(2,25)RKM,EINT,FI
 27   CONTINUE
      END
C
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1 +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1 -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1 -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is the
C coefficient matrix, from subroutine SPMAT, relating the
C second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second deivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1 +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2 +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END





 

























