C                           PROGRAMME FIGURE.FOR
C
C FIGURE.FOR calculates the mean density, the 'flattening', f, and
C departure from ellipticity, kappa, of the internal level surfaces for
C an input Earth model, under the hydrostatic assumption. Related
C second and fourth degree coefficients of the gravitational potential,
C J2 and J4, are also calculated.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Q(100),R(100),RHO(100),GZERO(100),RI(300),RHOI(300),
     1 GZEROI(300),ENAME(10),NM(4),NI(4),NK(4),RHOB(100),
     2 B(98,198), C(100,100),D(100,100,4),RHOBI(300),ETA(100),ETAI(300),
     3 E(100), EI(300),W(6),X(6),YYC(2,100),YYCI(2,300),YYP(2,100),
     4 YYPI(2,300),Y(2),YP(2),FI(300),YY(2,100),YYI(2,300),RHS(100)
      DOUBLE PRECISION MU(100),LAMBDA(100),MUI(300),LAMBDAI(300),
     1 INT(100,4),INTI(5,5),INITYC(2,5),INITYP(2,5),
     2 K1(2),K2(2),K3(2),K4(2),KAPPA,J2,J4
      CHARACTER*20 EMODEL
C Enter Earth model file name.
      WRITE(6,10)
 10   FORMAT(1x,'Type in Earth model file name.')
      READ(5,11)EMODEL
 11   FORMAT(A20)
C Open Earth model file.
      OPEN(UNIT=1,FILE=EMODEL,STATUS='OLD')
C Open figure file.
      OPEN(UNIT=2,FILE='figure.dat',STATUS='UNKNOWN')
C Open Inner Core data file.
       OPEN(UNIT=3,FILE='innerc.dat',STATUS='UNKNOWN')
C Set value of pi.
      PI=3.141592653589793D0
C Set value of universal constant of gravitation (CODATA 2006).
      G=6.67428D-11
C Set angular frequency of Earth's rotation (WGS84).
      WE=7.292115D-5
C Set maximum dimensions for interpolation.
      M1=100
      M2=198
      M3=98
C Read in and write out Earth model.
C Read in and write out headers.
C Earth model name.
      READ(1,12)(ENAME(I),I=1,10)
 12   FORMAT(10A7)
      WRITE(2,13)(ENAME(I),I=1,10)
      WRITE(3,12)(ENAME(I),I=1,10)
      WRITE(6,13)(ENAME(I),I=1,10)
 13   FORMAT(11X,10A7)
      READ(1,14)NN
 14   FORMAT(I10)
C NN is not used.
C Number of model points (NI is not used).
      READ(1,15)(NM(I),NI(I),I=1,4)
 15   FORMAT(8I10)
      WRITE(2,16)
      WRITE(6,16)
 16   FORMAT(/,/,5X,'Number of model points for inner core, outer core,
     1 mantle and crust;' ,/)
      WRITE(2,15)(NM(I),I=1,4)
      WRITE(6,15)(NM(I),I=1,4)
C Enter number of steps in each region for integrations.
      WRITE(6,17)
 17   FORMAT(/,/,1X,'Select number of steps per subinterval for integrat
     1ions. ')
      READ(5,*)NIS
      WRITE(2,18)NIS
 18   FORMAT(/,5X,'Number of steps per subinterval for integrations =',
     1 I3)
C Initialize model point count.
      K=0
C Read in Earth model.
      DO 19 M=1,4
        N1=NM(M)
        READ(1,20)(R(I),RHO(I),LAMBDA(I),MU(I),GZERO(I),I=1,N1)
 20     FORMAT(1X,F10.1,F10.2,F10.1,F10.1,F10.1)
        DO 21 I=1,N1
C Scale Earth model to SI values and store.
          J=K+I
          RI(J)=R(I)*1.D3
          RHOI(J)=RHO(I)*1.D3
          LAMBDAI(J)=LAMBDA(I)*1.D8
          MUI(J)=MU(I)*1.D8
          GZEROI(J)=GZERO(I)*1.D-2
 21     CONTINUE
C Record number of model points up to end of previous region.
        NK(M)=K
        K=K+N1
 19   CONTINUE
C Begin quadratures for solution of the Clairaut equation by
C the Radau transformation.
C Set weights and abscissae for six point Guassian integration.
      W(1)=0.171324492379170D0
      W(2)=0.360761573048139D0
      W(3)=0.467913934572691D0
      W(4)=W(3)
      W(5)=W(2)
      W(6)=W(1)
      X(1)=-0.9324695142031521D0
      X(2)=-0.6612093864662649D0
      X(3)=-0.2386191860831970D0
      X(4)=-X(3)
      X(5)=-X(2)
      X(6)=-X(1)
C Initialize starting values of the five quadratures at the geocentre.
      INTI(1,1)=0.D0
      INTI(2,1)=0.D0
      INTI(3,1)=0.D0
      INTI(4,1)=0.D0
      INTI(5,1)=0.D0 
      DO 22 M=1,4
        MP1=M+1
C Set up interpolation for region M.
        N1=NM(M)
        N2=2*N1-2
        N3=N1-2
C Put radius and density for region M in active locations.
        DO 23 I=1,N1
          J=NK(M)+I          
          R(I)=RI(J)
          RHO(I)=RHOI(J)
 23     CONTINUE
C Construct interpolation matrix.
        CALL SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C Store interpolation matrices.
        DO 24 I=1,N1
          DO 25 J=1,N1
            D(I,J,M)=C(I,J)
 25       CONTINUE
 24     CONTINUE 
C Begin quadrature for mean density.
C Set exponent in integrand.
        EX=2.D0
C Initialize starting value of integral INT.
        INT(1,M)=INTI(1,M)
C Form integrand function Q.
        DO 26 I=1,N1
          Q(I)=RHO(I)
 26     CONTINUE
C Perform quadrature.
        CALL QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C Store starting value for next region.
        INTI(1,MP1)=INT(N1,M)
C Form mean density and store in RHOBI.
        DO 27 I=1,N1
          IF(I.EQ.1.AND.M.EQ.1)GO TO 28
          RHOB(I)=3.D0*INT(I,M)/(R(I)**3)
          GO TO 29
 28       RHOB(I)=RHO(I)
C Store mean density.
 29       J=NK(M)+I
          RHOBI(J)=RHOB(I)   
 27     CONTINUE
C Perform first estimate of Radau variable ETA.
C Set exponent in integrand.
        EX=4.D0
C Initialize starting value of integral INT.
        INT(1,M)=INTI(2,M)
C Form integrand function Q.
        DO 30 I=1,N1
          Q(I)=RHOB(I)
 30     CONTINUE
C Perform quadrature.
        CALL QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C Store starting value for next region.
        INTI(2,MP1)=INT(N1,M)
C Form first estimate of ETA and store in ETAI.
        DO 31 I=1,N1
          IF(I.EQ.1.AND.M.EQ.1)GO TO 32
          ETA(I)=5.D0*INT(I,M)/(RHOB(I)*(R(I)**5))
          ETA(I)=ETA(I)*ETA(I)-1.D0
          GO TO 33
 32       ETA(I)=0.D0
C Store ETA.
 33       J=NK(M)+I
          ETAI(J)=ETA(I)
 31     CONTINUE
C Perform first estimate of figure parameter e.
C Set exponent in integral.
        EX=-1.D0
C Initialize starting value of integral INT.
        INT(1,M)=INTI(3,M)
C Form integrand function Q.
        DO 34 I=1,N1
          Q(I)=ETA(I)
 34     CONTINUE
C Perform quadrature.
        CALL QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C Store starting value for next region.
        INTI(3,MP1)=INT(N1,M)
C Form first estimate of e(r)/e(0) and store in EI.
        DO 35 I=1,N1
          IF(I.EQ.1.AND.M.EQ.1)GO TO 36
          E(I)=DEXP(INT(I,M))
          GO TO 37
 36       E(I)=1.D0
C Store e(r)/e(0) in EI.
 37       J=NK(M)+I
          EI(J)=E(I)
 35     CONTINUE
 22   CONTINUE
C Find total number of model points.
      NT=NK(4)+NM(4)
C Find surface value of Froude number or dimensionless parameter m(d).
      FRD=3.D0*WE*WE/(4.D0*PI*G*RHOBI(NT))
C Find surface value e(d).
C Generate coefficients of quadratic.
      BB=-(14.D0+7.D0*ETAI(NT)+6.D0*FRD)/4.D0
      CC=FRD*(35.D0/8.D0+5.D0*FRD/6.D0)
      ED=-(BB+DSQRT(BB*BB-4.D0*CC))/2.D0
C Find e(0).
      E0=ED/EI(NT)
C Find e(r).
      DO 38 I=1,NT
        EI(I)=E0*EI(I)
 38   CONTINUE
C Begin second pass on integration of the Clairaut equation
C by the Radau transformation.
      DO 39 M=1,4
        MP1=M+1
C Fix number of points for region M.
        N1=NM(M)
C Restore interpolation matrix for region M.
        DO 40 I=1,N1
          DO 41 J=1,N1
            C(I,J)=D(I,J,M)
 41       CONTINUE
 40     CONTINUE
C Put radius, density, mean density, ETA and e in
C active locations.
        DO 42 I=1,N1
          J=NK(M)+I
          R(I)=RI(J)
          RHO(I)=RHOI(J)
          RHOB(I)=RHOBI(J)
          ETA(I)=ETAI(J)
          E(I)=EI(J)
 42     CONTINUE
C Perform final estimate of Radau variable ETA.
C Set exponent in integrand.
        EX=4.D0
C Initialize starting value of integral INT.
        INT(1,M)=INTI(4,M)
C Form integrand function Q.
        DO 43 I=1,N1
C Calculate Froude number or dimensionless parameter m(r).
          FR=3.D0*WE*WE/(4.D0*PI*G*RHOB(I))
          Q(I)=7.D0*FR*(1.D0+ETA(I))-3.D0*E(I)*(1.D0+ETA(I))**2
     1    -4.D0*E(I)
          Q(I)=Q(I)*2.D0*(1.D0-RHO(I)/RHOB(I))/35.D0
          Q(I)=Q(I)+1.D0+ETA(I)/2.D0-ETA(I)*ETA(I)/10.D0
          Q(I)=Q(I)*RHOB(I)/DSQRT(1.D0+ETA(I))
 43     CONTINUE
C Perform quadrature.
        CALL QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C Store starting value for next region.
        INTI(4,MP1)=INT(N1,M)
C Form final estimate of ETA and store in ETA1.
        DO 44  I=1,N1
          IF(I.EQ.1.AND.M.EQ.1)GO TO 45
          ETA(I)=5.D0*INT(I,M)/(RHOB(I)*(R(I)**5))
          ETA(I)=ETA(I)*ETA(I)-1.D0
          GO TO 46
 45       ETA(I)=0.D0
C Store ETA.
 46       J=NK(M)+I
          ETAI(J)=ETA(I)
 44     CONTINUE
C Form final estimate of figure parameter e.
C Set exponent in integral.
        EX=-1.D0
C Initialize starting value of integral INT.
        INT(1,M)=INTI(5,M)
C Form integrand function Q.
        DO 47 I=1,N1
          Q(I)=ETA(I)
 47     CONTINUE
C Perform quadrature.
        CALL QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C Store starting value for next region.
        INTI(5,MP1)=INT(N1,M)
C Form final estimate of e(r)/e(0) and store in EI.
        DO 48 I=1,N1
          IF(I.EQ.1.AND.M.EQ.1)GO TO 49
          E(I)=DEXP(INT(I,M))
          GO TO 50
 49       E(I)=1.D0
C Store e(r)/e(0) IN EI.
 50       J=NK(M)+I
          EI(J)=E(I)
 48     CONTINUE  
 39   CONTINUE
C Find surface value e(d).
C Generate new coefficient of quadratic.
      BB=-(14.D0+7.D0*ETAI(NT)+6.D0*FRD)/4.D0
      ED=-(BB+DSQRT(BB*BB-4.D0*CC))/2.D0
C Find e(0).
      E0=ED/EI(NT)
C Find final e(r).
      DO 51 I=1,NT
        EI(I)=E0*EI(I)
 51   CONTINUE
C Begin calculation of kappa by Runge-Kutta integration.
      DO 52 M=1,4
         MP1=M+1
C Fix number of model points for region M.
         N1=NM(M)
C Restore interpolation matrix for region M.
         DO 53 I=1,N1
           DO 54 J=1,N1
             C(I,J)=D(I,J,M)
 54        CONTINUE
 53      CONTINUE
C Put radius, density, mean density, ETA and e in
C active locations.
         DO 55 I=1,N1
           J=NK(M)+I
           R(I)=RI(J)
           RHO(I)=RHOI(J)
           RHOB(I)=RHOBI(J)
           ETA(I)=ETAI(J)
           E(I)=EI(J)
 55      CONTINUE
C Begin calculation of the complementry function YYC.
C Set IC switch to unity for complimentary function.
         IC=1
C Set initial value of YYC vector at beginning of region.
         IF(M.EQ.1)GO TO 56
         YYC(1,1)=INITYC(1,M)
         YYC(2,1)=INITYC(2,M)
         GO TO 57
 56      YYC(1,1)=0.D0
         YYC(2,1)=0.D0
 57      CONTINUE
C Do Runge-Kutta extension of the complimentary function through region.
         CALL RK(YYC,IC,RHS,RHO,RHOB,E,NIS,N1,C,R,M,M1)
C Store starting value for next region.
         INITYC(1,MP1)=YYC(1,N1)
         INITYC(2,MP1)=YYC(2,N1)
C Store YYC in YYCI.
         DO 58 I=1,N1
           J=NK(M)+I
           YYCI(1,J)=YYC(1,I)
           YYCI(2,J)=YYC(2,I)
 58      CONTINUE
C Begin calculation of the particular integral YYP.
C Set switch IC to zero for particular integral.
         IC=0
C Set initial value of YYP vector at beginning of region.
         IF(M.EQ.1)GO TO 59
         YYP(1,1)=INITYP(1,M)
         YYP(2,1)=INITYP(2,M)
         GO TO 60
 59      YYP(1,1)=0.D0
         CALL DERIV(R(1),RHOP,N1,C,R,RHO,M1)
         YYP(2,1)=-RHOP*E(1)*E(1)/(64.D0*RHO(1))
 60      CONTINUE
C Begin Runge-Kutta extension of the particular integral through region.
C Construct right hand side.
         DO 61 I=1,N1
           IF(I.EQ.1.AND.M.EQ.1)GO TO 62
           RHS(I)=3.D0*E(I)*E(I)*(1.D0-RHO(I)/RHOB(I))*
     1     (4.D0+6.D0*ETA(I)+3.D0*ETA(I)*ETA(I))/(4.D0*R(I)*R(I))
     2     -E(I)*E(I)*(7.D0+5.D0*ETA(I))*ETA(I)/(2.D0*R(I)*R(I))
           GO TO 61
 62        RHS(1)=0.D0  
 61      CONTINUE
C Do Runge-Kutta extension.
         CALL RK(YYP,IC,RHS,RHO,RHOB,E,NIS,N1,C,R,M,M1)
C Store starting value for next region.
         INITYP(1,MP1)=YYP(1,N1)
         INITYP(2,MP1)=YYP(2,N1)  
C Store YYP in YYPI.
         DO 63 I=1,N1
           J=NK(M)+I
           YYPI(1,J)=YYP(1,I)
           YYPI(2,J)=YYP(2,I)
 63      CONTINUE
 52   CONTINUE
C Apply boundary condition to YYCI and YYPI.
C determine amplitude DD of YYCI at the surface.
      DD=-RI(NT)*YYPI(2,NT)-4.D0*YYPI(1,NT)
     1 -5.D0*FRD*(EI(NT)-5.D0*FRD/4.D0)/4.D0
      DD=DD/(RI(NT)*YYCI(2,NT)+4.D0*YYCI(1,NT))
C Find YYI and flattening FI.
      DO 64 I=1,NT
        YYI(1,I)=DD*YYCI(1,I)+YYPI(1,I)
        YYI(2,I)=DD*YYCI(2,I)+YYPI(2,I)
        FI(I)=EI(I)+5.D0*EI(I)*EI(I)/42.D0-4.D0*YYI(1,I)/7.D0
 64   CONTINUE
C Write out Inner Core data file, innerc.dat.
C Write out number of model points.
      WRITE(3,65)NM(1)
 65   FORMAT(1X,I9)
C Write out density at the bottom of the Outer Core.
      NM1P1=NM(1)+1
      WRITE(3,66)RHOI(NM1P1)
 66   FORMAT(1X,D15.8)
      WRITE(3,67)(RI(I),RHOI(I),RHOBI(I),FI(I),I=1,NM(1))
 67   FORMAT(1X,4D15.8)
C Write out radius, density, mean density, 1/f and kappa.
C Write out headings.
      WRITE(2,68)
      WRITE(6,68)
 68   FORMAT(/,/,10X,'Radius',8X,'Density',6X,'Mean Density',6X,'1/f',
     1 11 X ,'kappa ')
      WRITE(2,69)
      WRITE(6,69)
 69   FORMAT(11X,'(km)',9X,'(gm/cc)',8X,'(gm/cc)',4X,/)
C Scale and write out values.
      DO 70 I=1,NT
        RAD=RI(I)/1000.D0
        DEN=RHOI(I)/1000.D0
        DENB=RHOBI(I)/1000.D0
        RECIPF=1.D0/FI(I)
        KAPPA=YYI(1,I)
        WRITE(2,71)RAD,DEN,DENB,RECIPF,KAPPA
        WRITE(6,71)RAD,DEN,DENB,RECIPF,KAPPA
 71     FORMAT(1X,F15.1,3F15.3,0PD15.3)
 70   CONTINUE
C Calculate values of second and fourth degree gravitational coefficients,
C J2, J4, and write out values.
      J2=-FRD/3.D0+2.D0*FI(NT)/3.D0+2.D0*FRD*FI(NT)/21.D0
     1-FI(NT)*FI(NT)/3.D0+8.D0*YYI(1,NT)/21.D0
      J4=4.D0*FRD*FI(NT)/7.D0-4.D0*FI(NT)*FI(NT)/5.D0
     1-32.D0*YYI(1,NT)/35.D0
      WRITE(6,72)J2
      WRITE(2,72)J2
 72   FORMAT(/,/,5X,'J2=',4PD14.7,/)
      WRITE(6,73)J4
      WRITE(2,73)J4
 73   FORMAT(/,/,5X,'J4=',1PD11.4,/)
      END
C
      SUBROUTINE QUAD(INT,EX,Q,NIS,W,X,N1,C,R,M,M1)
C
C This subroutine performs the quadrature integral of a function, Q, of mean
C radius, from the centre of the Earth through the Inner Core for M=1, or the
C Outer Core for M=2, or the Mantle for M=3, or the Crust for M=4. The
C result is returned in INT. The integration uses six point Guassian integration
C which is eleventh order accurate (exact for polynomials up to and including
C degree eleven). Interpolation of Q to the Guassian integration points is
C performed by the spline interpolator subroutine INTPL. The number of
C integration steps between Earth model points is specified by NIS.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Q(M1),R(M1),C(M1,M1),W(6),X(6),RI(6),
     1 QI(6)
      DOUBLE PRECISION INT(M1,4)
      N1M1=N1-1
      DO 10 I=1,N1M1
        IP1=I+1
C Find half increment in radius.
        H=(R(IP1)-R(I))/(2.D0*DFLOAT(NIS))
C Set sum to zero.
        SUM=0.D0
C Integrate over subinterval.
        DO 11 J=1,NIS
          JM1=J-1 
C Find mean radius.
          RM=R(I)+2.D0*DFLOAT(JM1)*H+H
C Find radii for Guassian integration.
          DO 12 K=1,6
            RI(K)=H*X(K)+RM
C Interpolate integrand at Guassian abscissae.
            CALL INTPL(RI(K),QI(K),N1,C,R,Q,M1)
C Accumulate sum.
            SUM=SUM+W(K)*QI(K)*((RI(K))**EX)
 12       CONTINUE
 11     CONTINUE
C Find contribution of current increment.
        INT(IP1,M)=INT(I,M)+H*SUM
 10   CONTINUE
      RETURN
      END
C
      SUBROUTINE DYDR(YP,RAD,Y,IC,RHS,RHO,RHOB,E,N1,C,R,M,M1)
C
C DYDR calculates the derivative YP of the vector Y at radius RAD. For IC=1,
C the derivative of the complimentary function for the kappa equation is found.
C For IC=0, the derivative of the particular integral solution of the kappa equation
C is found. The special cases of the derivatives at the geocentre are treated separately.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION YP(2),Y(2),R(M1),C(M1,M1),RHS(M1),RHO(M1),
     1 RHOB(M1),E(M1)
      IF(RAD.EQ.0.D0)GO TO 10
      YP(1)=Y(2)
C Test complimentary function.
      IF(IC.EQ.1)GO TO 11
C Interpolate RHS to get right hand side RS at radius RAD.
      CALL INTPL(RAD,RS,N1,C,R,RHS,M1)
C Interpolate RHO and RHOB at radius RAD.
 11   CALL INTPL(RAD,DEN,N1,C,R,RHO,M1)
      CALL INTPL(RAD,DENB,N1,C,R,RHOB,M1)
      YP(2)=-(6.D0*DEN/DENB-20.D0)*Y(1)/(RAD*RAD)
     1 -6.D0*DEN*Y(2)/(RAD*DENB)
      IF(IC.EQ.0)YP(2)=YP(2)+RS 
      GO TO 12
C Test for particular integral.
 10   IF(IC.EQ.0)GO TO 13
      YP(1)=0.D0
      YP(2)=2.D0
      GO TO 12
 13   CALL DERIV(RAD,RHOP,N1,C,R,RHO,M1)
      YP(1)=-RHOP*E(1)*E(1)/(64.D0*RHO(1))
      YP(2)=0.D0
 12   CONTINUE
      RETURN
      END
C
      SUBROUTINE RK(YY,IC,RHS,RHO,RHOB,E,NIS,N1,C,R,M,M1)
C
C This subroutine carries out a fourth oder Runge-Kutta integration of the
C second order equation for kappa, giving the departure of the internal
C figures of equilibrium from the ellipsoid. For M=1 the integration
C is through the Inner Core, for M=2 the integration is through the
C Outer Core, for M=3 the integration is through the Mantle, and for
C M=4 the integration is through the Crust. The number of integration
C steps between Earth model points is specified by NIS.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION RHS(M1),YY(2,M1),R(M1),C(M1,M1),Y(2),YP(2),SUM(2),
     1 RHO(M1),RHOB(M1),E(M1)
      DOUBLE PRECISION K1(2),K2(2),K3(2),K4(2)
      N1M1=N1-1
      DO 10 I=1,N1M1
        IP1=I+1
C Determine step size.
        H=(R(IP1)-R(I))/(DFLOAT(NIS))
C Determine initial radius of step.
        RAD=R(I)
C Set cumulative sums to zero.
        SUM(1)=0.D0
        SUM(2)=0.D0
        DO 11 J=1,NIS
C Determine current Y vector.
          Y(1)=YY(1,I)+SUM(1)
          Y(2)=YY(2,I)+SUM(2)       
          CALL DYDR(YP,RAD,Y,IC,RHS,RHO,RHOB,E,N1,C,R,M,M1)
C Find vector K1.
          K1(1)=H*YP(1)
          K1(2)=H*YP(2)
C Increment RAD and Y.
          RAD=RAD+H/2.D0
          Y(1)=Y(1)+K1(1)/2.D0
          Y(2)=Y(2)+K1(2)/2.D0
          CALL DYDR(YP,RAD,Y,IC,RHS,RHO,RHOB,E,N1,C,R,M,M1)
C Find vector K2.
          K2(1)=H*YP(1)
          K2(2)=H*YP(2)
C Increment Y.
          Y(1)=Y(1)+K2(1)/2.D0
          Y(2)=Y(2)+K2(2)/2.D0
          CALL DYDR(YP,RAD,Y,IC,RHS,RHO,RHOB,E,N1,C,R,M,M1)
C Find vector K3.
          K3(1)=H*YP(1)
          K3(2)=H*YP(2)
C Increment RAD and Y.
          RAD=RAD+H/2.D0
          Y(1)=Y(1)+K3(1)
          Y(2)=Y(2)+K3(2)
          CALL DYDR(YP,RAD,Y,IC,RHS,RHO,RHOB,E,N1,C,R,M,M1)
C Find vector K4.
          K4(1)=H*YP(1)
          K4(2)=H*YP(2)
C Accumulate sums.
          SUM(1)=SUM(1)+(K1(1)+2.D0*K2(1)+2.D0*K3(1)+K4(1))/6.D0
          SUM(2)=SUM(2)+(K1(2)+2.D0*K2(2)+2.D0*K3(2)+K4(2))/6.D0
 11     CONTINUE
C Increment YY.
        YY(1,IP1)=YY(1,I)+SUM(1)
        YY(2,IP1)=YY(2,I)+SUM(2)
 10   CONTINUE
      RETURN
      END
C  
      SUBROUTINE SPMAT(N1,N2,N3,C,R,B,M1,M2,M3)
C
C SPMAT computes the N1 by N1 matrix C connecting
C second derivatives to function values at N1 nodal points R(I).
C B is the N3=N1-2 by N2=2*N1-2 augmented matrix used internally
C in the Gaussian elimination procedure.
C M1, M2, M3 are the respective maximum dimensions for N1, N2, N3
C specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),B(M3,M2)
      N=N3+1
C Construct augmented matrix A.
      DO 11 I=1,N3
        DO 10 J=1,N2
          B(I,J)=0.D0
  10    CONTINUE
        B(I,I)=2.D0*(R(I+2)-R(I))
        B(I,I+N3)=1.D0
        IF(I.EQ.1.OR.I.EQ.N3) GO TO 11
        B(I,I-1)=R(I+1)-R(I)
        B(I,I+1)=R(I+2)-R(I+1)
  11  CONTINUE
C Apply conditions that third derivatives are constant
C in first two and last two intervals.
      B(1,2)=R(3)-R(2)-((R(2)-R(1))**2.D0)/(R(3)-R(2))
      B(N3,N3-1)=R(N3+1)-R(N3)-((R(N1)-R(N1-1))**2.D0)/(R(N1-1)-R(N1-2))
      B(1,1)=B(1,1)+((R(3)-R(1))*(R(2)-R(1)))/(R(3)-R(2))
      B(N3,N3)=B(N3,N3)
     1 +((R(N1)-R(N1-1))*(R(N1)-R(N1-2)))/(R(N1-1)-R(N1-2))
      NM1=N3-1
C Eliminate sub-diagonal.
      DO 13 I=1,NM1
        DIAG=B(I,I)
        ELL=B(I+1,I)
        DO 12 J=1,N2
          B(I,J)=B(I,J)/DIAG
          B(I+1,J)=B(I+1,J)-B(I,J)*ELL
  12    CONTINUE
  13  CONTINUE
C Eliminate super-diagonal.
      DO 15 I=1,NM1
        NO=N-I
        DIAG=B(NO,NO)
        ELL=B(NO-1,NO)
        DO 14 J=1,N2
          B(NO,J)=B(NO,J)/DIAG
          B(NO-1,J)=B(NO-1,J)-B(NO,J)*ELL
  14    CONTINUE
  15  CONTINUE
C Move inverse to position formerly occupied by A.
      DO 17 I=1,N3
        DO 16 J=1,N3
          B(I,J)=B(I,J+N3)
  16    CONTINUE
  17  CONTINUE
C Construct matrix B in N-2 by N augmentation space. 
      DO 19 I=1,N3
        DO 18 J=1,N1
          B(I,J+N3)=0.0D0
  18    CONTINUE
        IN=I+N3
        FACTOR=6.D0/((R(I+2)-R(I+1))*(R(I+1)-R(I)))
        B(I,IN)=FACTOR*(R(I+2)-R(I+1))
        B(I,IN+1)=-FACTOR*(R(I+2)-R(I))
        B(I,IN+2)=FACTOR*(R(I+1)-R(I))
  19  CONTINUE
C Multiply inverse of A into B to get N-2 by N version of C.
      DO 22 I=1,N3
        DO 21 J=1,N1
          SUM=0.D0
          DO 20 K=1,N3
            SUM=SUM+B(I,K)*B(K,J+N3)
  20      CONTINUE
          C(I+1,J)=SUM
  21    CONTINUE
  22  CONTINUE
C Add first and last rows to C.
      DO 23 J=1,N1
        C(1,J)=C(2,J)*(R(3)-R(1))/(R(3)-R(2))
     1 -C(3,J)*(R(2)-R(1))/(R(3)-R(2))
        C(N1,J)=C(N1-1,J)*(R(N1)-R(N1-2))/(R(N1-1)-R(N1-2))
     1 -C(N1-2,J)*(R(N1)-R(N1-1))/(R(N1-1)-R(N1-2))
  23  CONTINUE
      RETURN
      END
C
      SUBROUTINE INTPL(XV,YV,N1,C,R,S,M1)
C
C INTPL interpolates to find YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second deivative SUM2 at end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE
C Find interpolate YV at XV.
      YV=S(J)+(S(J+1)-S(J))*(XV-R(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     2  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
C
      SUBROUTINE DERIV(XV,YV,N1,C,R,S,M1)
C
C DERIV calculates the derivative YV at XV, using a cubic spline,
C given the function S(I) at N1 nodal points R(I). C is
C the coefficient matrix, from subroutine SPMAT, relating
C the second derivatives to the function values at the nodal points.
C M1 is the maximum dimension for N1 specified in the main programme.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION C(M1,M1),R(M1),S(M1)
      NPM1=N1-1
      J=NPM1
C Locate subinterval.
      DO 10 L=1,NPM1
        IF(XV.GE.R(L).AND.XV.LT.R(L+1)) J=L
  10  CONTINUE
C Find second derivative SUM1 at beginning of subinterval
C and second derivative SUM2 at the end of subinterval.
      SUM1=0.D0
      SUM2=0.D0
      DO 11 K=1,N1
        SUM1=SUM1+C(J,K)*S(K)
        SUM2=SUM2+C(J+1,K)*S(K)
  11  CONTINUE   
C Find derivative YV at XV.
      YV=(S(J+1)-S(J))/(R(J+1)-R(J))
     1  +(XV-R(J))*(XV-R(J+1))*(SUM2-SUM1)/(6.D0*(R(J+1)-R(J)))
     2  +(2.D0*XV-R(J+1)-R(J))*(SUM1*(2.D0-(XV-R(J))/(R(J+1)-R(J)))
     3  +SUM2*(1.D0+(XV-R(J))/(R(J+1)-R(J))))/6.D0
      RETURN
      END
