C           PROGRAMME SVDDFT.FOR                                        
C
C SVDDFT.FOR finds the DFT of an unequally spaced time sequence of VLBI
C polar motion or nutation observations using singular value decomposition
C (SVD) combined with Parseval's theorem to determine the number of
C singular values to eliminate (NSVE). The sequence is reduced to zero mean,
C and it is then Parzen windowed by a call to the subroutine WINDOW
C to supress finite record effects. The SVD is found by a call to
C the subroutine SVD which in turn calls the subroutines HHOLDER and BIDIAG.
C The programme allows iteration to find the optimum value of NSVE
C to satisfy Parseval's theorem.        
      IMPLICIT DOUBLE COMPLEX(A-H,O-Z)
      DOUBLE PRECISION SIGS(4000),DAY(4000),S(4000),XWOB,YWOB,EXWOB,
     1 EYWOB,SVL,T,T0,DT,AM,AN,AJ1,AJ2,ARG,ARG1,ARG2,RMS,SUMX,SUMY,
     2 AAI,ERROR,AJ,FREQ,W,PI,SUMS,OMEGA,TD
      DIMENSION G(4000),GP(4000),A(4000,4000),SV(4000),U(4000,4000),
     1 US(4000,4000),VH(4000,4000),B(4000)
      CHARACTER*20 INSEQ,DFTOUT
C Type in name of input sequence file.
      WRITE(6,10)
 10   FORMAT(1X,'Type in name of input sequence file.')
      READ(5,11)INSEQ
 11   FORMAT(A20)
C Type in name of output DFT file.
      WRITE(6,12)
 12   FORMAT(1X,'Type in name of output DFT file.')
      READ(5,11)DFTOUT
      OPEN(UNIT=1,FILE=INSEQ,STATUS='OLD')
      OPEN(UNIT=2,FILE='err.dat',STATUS='UNKNOWN')
      OPEN(UNIT=3,FILE=DFTOUT,STATUS='UNKNOWN')
      OPEN(UNIT=4,FILE='sv.dat',STATUS='UNKNOWN')
      OPEN(UNIT=7,FILE='rec.dat',STATUS='UNKNOWN')
C Set values of pi and 2pi.
      PI=3.141592653589793D0
      OMEGA=2.D0*PI
C Enter segment length and time at segment midpoint, from beginning of record.
      WRITE(6,13)
 13   FORMAT(1X,'Enter segment length and time at segment midpoint.')
      READ(5,*)T,T0
C Enter first and last point numbers of segment data set.
      WRITE(6,14)
 14   FORMAT(1X,'Enter first and last point numbers of segment.')
      READ(5,*)IB,IE
C Calculate length of data set N.
      N=IE-IB+1
C Find length of DFT, M, an odd integer less than or equal to N.
      M=N
      NB2=N/2 
      IF(N.EQ.2*NB2)M=N-1
C Find number of positive harmonics.
      MP=(M-1)/2
C Find equivalent time sample interval.
      AM=DFLOAT(M)
      DT=T/AM
C Read in unused initial record segment.
      IBM1=IB-1
      DO 15 I=1,IBM1
        READ(1,16)K,IYYY,MM,ID,TD,XWOB,YWOB,EXWOB,EYWOB
 16     FORMAT(1X,4I5,F10.6,4F11.6)
 15   CONTINUE
C Set sums of real and imaginary parts of co-ordinates to zero.
      SUMX=0.D0
      SUMY=0.D0
C Read in segment data set.
      DO 17 I=1,N
        READ(1,16)K,IYYY,MM,ID,DAY(I),XWOB,YWOB,EXWOB,EYWOB
C Accumulate sums of real and imaginary parts of co-ordinates.
        SUMX=SUMX+XWOB
        SUMY=SUMY+YWOB  
C Convert co-ordinates to complex form.
        G(I)=DCMPLX(XWOB,YWOB)
C Find variance.
        SIGS(I)=EXWOB*EXWOB+EYWOB*EYWOB
 17   CONTINUE  
C Find means of real and imaginary parts of co-ordinates.
      AN=DFLOAT(N)
      SUMX=SUMX/AN
      SUMY=SUMY/AN
C Reduce co-ordinates to zero mean and calculate their sum of squares.
      RMS=0.D0
      DO 18 I=1,N
        G(I)=G(I)-DCMPLX(SUMX,SUMY)
        RMS=RMS+G(I)*DCONJG(G(I))
C Window the data segment with the Parzen window.
        CALL WINDOW(DAY(I),T0,T,W)
        G(I)=G(I)*W
 18   CONTINUE  
C Find left side of Parseval's relation.
      SUMS=RMS*DT
C Find root mean square of signal.
      RMS=RMS/AN
      RMS=DSQRT(RMS) 
C Construct first column, A(J,1), of Toeplitz conditional equations matrix,
C and right-hand side vector B(J).
      DO 19 J=1,M
        AJ1=DFLOAT(J-1)
        AJ2=DFLOAT(J-MP-1)
        A(J,1)=(0.D0,0.D0)
        B(J)=(0.D0,0.D0)
        DO 20 I=1,N
C Form arguments of complex exponentials.
          ARG=-2.D0*PI*DAY(I)/T
          ARG1=ARG*AJ1
          ARG2=ARG*AJ2
C Perform sums.
          A(J,1)=A(J,1)+DCMPLX(DCOS(ARG1),DSIN(ARG1))/SIGS(I)
          B(J)=B(J)+G(I)*DCMPLX(DCOS(ARG2),DSIN(ARG2))/SIGS(I)
 20    CONTINUE
        B(J)=T*B(J)
 19   CONTINUE
C Complete construction of conditional equations matrix A.
      DO 21 J=2,M
        DO 23 I=1,M
          K=I-J+1
          L=2-K
          IF(K.LE.0)GO TO 22
          A(I,J)=A(K,1)
          GO TO 23
 22       A(I,J)=DCONJG(A(L,1))
 23    CONTINUE
 21   CONTINUE
C Do singular value decomposition of conditional equations matrix A=U*S*VH.
      CALL SVD(A,U,S,VH,M,4000)
C Multiply the Hermitian transpose of U into right-hand side vector B.
      DO 24 I=1,M
C Accumulate product vector in SV.
        SV(I)=(0.D0,0.D0)
        DO 25 J=1,M
          SV(I)=SV(I)+DCONJG(U(J,I))*B(J)
 25    CONTINUE
 24   CONTINUE
C Replace right-hand side vector B by product vector SV.
      DO 26 I=1,M
        B(I)=SV(I)
 26   CONTINUE
C Write out singular values.
      WRITE(4,27)
 27   FORMAT(1X,'Singular values are:')
      WRITE(4,28)(S(I),I=1,M)
 28   FORMAT(1X,D23.15)
C Begin solution of reduced conditional equations.
C Set number of singular values to be eliminated.
      WRITE(6,29)
 29   FORMAT(1X,'Enter number of singular values to be eliminated.')
      READ(5,*)NSVE
C Find number of remaining singular values.
 30   NMAX=M-NSVE
C Multiply V into the inverse of truncated S.
      DO 31 I=1,M
        DO 32 J=1,M  
          IF(J.GT.NMAX)GO TO 33
          US(I,J)=DCONJG(VH(J,I))/S(J)
          GO TO 32
 33       US(I,J)=(0.D0,0.D0)
 32    CONTINUE
 31   CONTINUE
C Multiply result into modified right-hand side to find solution vector SV
C and solution vector length SVL.
      SVL=0.D0
      DO 34 I=1,M
        SV(I)=(0.D0,0.D0)
        DO 35 J=1,M
          SV(I)=SV(I)+US(I,J)*B(J)
 35    CONTINUE
        SVL=SVL+SV(I)*DCONJG(SV(I))
 34   CONTINUE
C Divide by segment length to get right-hand side of Parseval's relation,
C then divide by left side of Parseval's relation to convert SVL to
C Parseval's ratio.
      SVL=SVL/(T*SUMS)
C Construct approximation, GP, to original data set G, from DFT SV.
      DO 36 J=1,N
        GP(J)=(0.D0,0.D0)
        DO 37 I=1,M
          AAI=DFLOAT(I-MP-1)
          ARG=OMEGA*AAI*DAY(J)/T
          GP(J)=GP(J)+SV(I)*DCMPLX(DCOS(ARG),DSIN(ARG))
 37    CONTINUE
C Normalize reconstructed data set.
        GP(J)=GP(J)/T
 36   CONTINUE
C Write out original and reconstructed data sets and calculate mean square
C error in reconstruction ERROR.
      ERROR=0.D0
      DO 38 J=1,N
        ERRORC=G(J)-GP(J)
        ERROR=ERROR+ERRORC*DCONJG(ERRORC)
        WRITE(7,39)DAY(J),G(J),GP(J)
 39     FORMAT(1X,F12.4,4D15.7)
 38   CONTINUE
C Find mean square error.
      ERROR=ERROR/AN
C Find relative reconstruction error.
      ERROR=DSQRT(ERROR)/RMS
C Write out number of singular values eliminated (NSVE),
C relative reconstruction error (RRE), and Parseval's ratio (PR). 
      WRITE(2,40)NSVE,ERROR,SVL
      WRITE(6,40)NSVE,ERROR,SVL
 40   FORMAT(1X,I10,2D15.8)
C Decide whether to try new value of NSVE or to end programme.
      WRITE(6,41)                                                             
 41   FORMAT(1X,'Enter 1 for new NSVE or 0 to end.')
      READ(5,*)ISWITCH
      IF(ISWITCH.NE.0)WRITE(6,42)
      IF(ISWITCH.EQ.0)GO TO 43
 42   FORMAT(1X,'Enter new value of NSVE.')
      READ(5,*)NSVE
      GO TO 30
 43   CONTINUE
C Write out recovered DFT.
      DO 44 J=1,M
        AJ=DFLOAT(J-MP-1)
        FREQ=AJ/T
        WRITE(3,45)FREQ,SV(J)
 45     FORMAT(1X,D15.5,2D26.15)
 44   CONTINUE
      END
C
      SUBROUTINE WINDOW(T,T0,AM,W)
C
C WINDOW finds the value of the PARZEN window, W, at time T,
C for a window of length AM centred at time T0.
C 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C Find relative time on (-1,1) time base.
      X=2.D0*(T-T0)/AM
C Find unity minus absolute value of relative time.
      OMX=1.D0-DABS(X)
C If absolute value of relative time is greater than 0.5,
C select alternate cubic.
      IF(DABS(X).GT.0.5D0) GO TO 10
      W=1.D0-6.D0*OMX*X*X
      GO TO 20
 10   W=2.D0*OMX*OMX*OMX
 20   CONTINUE   
      RETURN
      END
C
      SUBROUTINE SVD(C,U,S,VH,M,MMAX)
C
C The subroutine SVD calls the subroutine BIDIAG to convert the complex
C matrix C to real, positive, upper bidiagonal form. The diagonal elements
C are stored in the vector S and its upper diagonal is stored in
C the vector UD. An implicit QR iteration with Wilkinson shift is then
C performed on the bidiagonal matrix through a series of Givens rotations
C to chase a rogue element, RE, from the matrix. After each QR iteration,
C the upper diagonal element in the leading row is tested for negligibility.
C The matrix is deflated when this element is found negligible, to
C computational accuracy, compared to its smallest diagonal neighbour,
C establishing a new leading row K. Similarly, a new trailing row, N, is
C established as the first row below the leading row with an upper diagonal
C element that is neglible compared to its smallest diagonal neighbour.
C The final iteration is on a 2 x 2 matrix. The singular values are on
C the diagonal of the final matrix. The singular values and their
C asociated eigenvectors are then sorted so that they are in descending
C order down the diagonal of the final matrix. The Hermitian transposes of
C the accumulated matrices U and VH are taken to convert them to
C the matrices appearing in the SVD factorization.
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DOUBLE COMPLEX C(MMAX,MMAX),U(MMAX,MMAX),VH(MMAX,MMAX),HOLD
      DIMENSION S(MMAX),UD(MMAX)
C Set maximum number of iterations.
      MAXI=50
      CALL BIDIAG(U,C,VH,M,MMAX)
C Put diagonal of real, positive bidiagonal matrix in S and its upper
C diadgonal in UD.
      DO 10 I=1,M
        IP1=I+1 
        S(I)=DREAL(C(I,I))
        IF(IP1.GT.M) GO TO 10
        UD(I)=DREAL(C(I,IP1))
 10   CONTINUE
C Begin diagonalization of matrix B.
C Set initial values of iteration number, ITER, starting row, K, and
C trailing row, N, of working matrix.
      ITER=0
      K=1
      N=M
C Continue reduction of working matrix.
 11   CONTINUE
C Advance iteration counter.
      ITER=ITER+1
      KP1=K+1
      NM1=N-1
C Find Wilkinson shift.
      P=S(N)
      Q=S(NM1)
      R=UD(NM1)
      H1=DSQRT((P+Q)*(P+Q)+R*R)
      H2=DSQRT((P-Q)*(P-Q)+R*R)
      SIG1=(H1-H2)/2.D0
C Find first Givens rotation.
      T11=(S(K)-SIG1)*(S(K)+SIG1)
      T12=S(K)*UD(K)
      CMAG=DSQRT(T11*T11+T12*T12)
      CO=T11/CMAG
      SI=-T12/CMAG
C Update unitary matrix VH.
      DO 12 I=1,M
        HOLD=VH(I,K)
        VH(I,K)=CO*HOLD-SI*VH(I,KP1)
        VH(I,KP1)=SI*HOLD+CO*VH(I,KP1)
 12   CONTINUE
C Post-multiply bidiagonal matrix B by rotation.
      H1=CO*S(K)-SI*UD(K)
      H2=CO*UD(K)+SI*S(K)
      S(K)=H1
      UD(K)=H2
C Find rogue element RE.
      RE=-SI*S(KP1)
      S(KP1)=CO*S(KP1)
C Find remaining Givens rotations.
      DO 13 I=K,NM1
      IP1=I+1
      IP2=I+2
      CMAG=DSQRT(S(I)*S(I)+RE*RE)
      CO=S(I)/CMAG
      SI=-RE/CMAG
C Update unitary matrix U.
      DO 14 J=1,M
        HOLD=U(I,J)
        U(I,J)=CO*HOLD-SI*U(IP1,J)
        U(IP1,J)=SI*HOLD+CO*U(IP1,J)
 14   CONTINUE
C Pre-multiply matrix B by rotation.
      H1=CMAG
      H2=CO*UD(I)-SI*S(IP1)
      H4=CO*S(IP1)+SI*UD(I)
      S(I)=H1
      UD(I)=H2
      S(IP1)=H4
      IF(IP2.GT.N) GO TO 13
      RE=-SI*UD(IP1)
      UD(IP1)=CO*UD(IP1)
C Find next Givens rotation.
      CMAG=DSQRT(UD(I)*UD(I)+RE*RE)
      CO=UD(I)/CMAG
      SI=-RE/CMAG
C  Update unitary matrix VH.
      DO 15 J=1,M
        HOLD=VH(J,IP1)
        VH(J,IP1)=CO*HOLD-SI*VH(J,IP2)
        VH(J,IP2)=SI*HOLD+CO*VH(J,IP2)
 15   CONTINUE
C Post-multiply matrix B by rotation.
      H1=CMAG
      H2=CO*S(IP2)
      H3=CO*S(IP1)-SI*UD(IP1)
      H4=CO*UD(IP1)+SI*S(IP1)
      UD(I)=H1
      S(IP1)=H3
      UD(IP1)=H4
      RE=-SI*S(IP2)
      S(IP2)=H2
 13   CONTINUE
C Find leading row of working matrix.
      J=K
      MM1=M-1
      DO 16 I=K,MM1
        IP1=I+1
C Find smallest neighbouring diagonal element.
        SMIN=DMIN1(DABS(S(I)),DABS(S(IP1)))
C Compare to see if upper diagonal element is negligible to machine precision.
        SIZE=SMIN+DABS(UD(I))
        IF(SIZE.NE.SMIN) GO TO 17
        J=J+1
 16   CONTINUE
 17   CONTINUE
C Set new leading row number.
      K=J
C Test to see if diagonalization is complete.
      IF(K.EQ.M) GO TO 20
C Find trailing row of working matrix.
      J=K+1
      KKP1=K+1
      DO 18 I=KKP1,MM1
        IP1=I+1
C Find smallest neighbouring diagonal element.
        SMIN=DMIN1(DABS(S(I)),DABS(S(IP1)))
C Compare to see if upper diagonal element is negligible.
        SIZE=SMIN+DABS(UD(I))
        IF(SIZE.EQ.SMIN) GO TO 19
        J=J+1
 18   CONTINUE
 19   CONTINUE
C Set new trailing row number.
      N=J
C Test for failure to converge.
      NNM1=N-1
      IF(KKP1.NE.KP1.OR.NNM1.NE.NM1) ITER=0
      IF(ITER.EQ.MAXI) PAUSE 'Maximum iterations reached.'
      GO TO 11
 20   CONTINUE
C Take Hermitian transpose of U and VH to get their final forms.
      DO 21 I=1,M
        DO 21 J=1,I
          HOLD=U(I,J)
          U(I,J)=DCONJG(U(J,I))
          U(J,I)=DCONJG(HOLD)
 21   CONTINUE
      DO 22 I=1,M
        DO 22 J=1,I
          HOLD=VH(I,J)
          VH(I,J)=DCONJG(VH(J,I))
          VH(J,I)=DCONJG(HOLD)
 22   CONTINUE
C Sort singular values into descending order, exchange columns of U
C and rows of VH.
      N=1
 23   CONTINUE
      IMAX=N
      DO 24 I=N,M
        IF(S(I).GE.S(IMAX)) IMAX=I
 24   CONTINUE
C Exchange singular values.
      HOLD=S(N)
      S(N)=S(IMAX)
      S(IMAX)=HOLD
C Exchange columns of U and rows of VH.
      DO 25 I=1,M
        HOLD=U(I,N)
        U(I,N)=U(I,IMAX)
        U(I,IMAX)=HOLD
        HOLD=VH(N,I)
        VH(N,I)=VH(IMAX,I)
        VH(IMAX,I)=HOLD
 25   CONTINUE  
      N=N+1
      IF(N.LE.M) GO TO 23    
      RETURN    
      END
C
      SUBROUTINE BIDIAG(U,C,VH,M,MMAX)
C
C The subroutine BIDIAG converts the complex M x M matrix C into upper
C bidiagonal form by a series of Householder unitary transformations,
C applied alternately, to columns by pre-multiplication and to rows by
C post-multiplication. Finally, rows are scaled by pre-multiplication
C with a unitary matrix and, alternately, columns are scaled by
C post-multiplication with a unitary matrix, to reduce the phase of diagonal
C elements to zero and, alternately, to reduce the phase of above diagonal
C elements to zero, producing an upper bidiagonal matrix with real and
C positive elements. Thus, the final, real and positive upper bidiagonal
C matrix B is found as B=U.C.VH, a unitary similarity transformation of C.
C B is returned in C. The unitary matrices U and VH are also returned by
C the subroutine.
C 
      DOUBLE COMPLEX C(MMAX,MMAX),V(MMAX),HU(MMAX),
     1U(MMAX,MMAX),VH(MMAX,MMAX),H0(MMAX),H1(MMAX),H2(MMAX),CONST
      DOUBLE PRECISION EMAG 
C Initialize unitary matrices U and VH.
      DO 10 I=1,M
        DO 11 J=1,M
          U(I,J)=(0.D0,0.D0)
          VH(I,J)=(0.D0,0.D0)
 11     CONTINUE
        U(I,I)=(1.D0,0.D0)
        VH(I,I)=(1.D0,0.D0)  
 10   CONTINUE  
C Apply Householder annihilation.
      MM1=M-1
      DO 12 N=1,MM1
C Put Nth column in V.
        DO 13 I=1,M
          V(I)=C(I,N)
 13     CONTINUE
C Annihilate Nth column.  
        CALL HHOLDER(V,HU,N,M,MMAX)
C Multiply Hermitian transpose of Householder vector HU
C into unitary matrix U and into transformed matrix B.
        DO 14 I=1,M
          H1(I)=(0.D0,0.D0)
          H2(I)=(0.D0,0.D0)
          DO 14 J=N,M
            H0(J)=DCONJG(HU(J))
            H1(I)=H1(I)+H0(J)*U(J,I)
            H2(I)=H2(I)+H0(J)*C(J,I)
 14     CONTINUE
C Update unitary matrix U and transformed matrix B.
        DO 15 I=N,M
          DO 15 J=1,M
            U(I,J)=U(I,J)-2.D0*HU(I)*H1(J)
            C(I,J)=C(I,J)-2.D0*HU(I)*H2(J)
 15     CONTINUE
C Overwrite annihilated Nth column.
        DO 16 I=1,M
          C(I,N)=V(I)
 16     CONTINUE  
        IF(N.EQ.MM1) GO TO 12       
C Put Nth row in V.
        DO 17 J=1,M
          V(J)=C(N,J)
 17     CONTINUE
        NP1=N+1
C Annihilate Nth row.
        CALL HHOLDER(V,HU,NP1,M,MMAX)
C Multiply unitary matrix VH and transformed matrix C
C into Householder vector HU.
        DO 18 I=1,M
          H1(I)=(0.D0,0.D0)
          H2(I)=(0.D0,0.D0)
          DO 18 J=NP1,M
            H0(J)=DCONJG(HU(J))
            H1(I)=H1(I)+VH(I,J)*H0(J)
            H2(I)=H2(I)+C(I,J)*H0(J)
 18     CONTINUE
C Update unitary matrix VH and transformed matrix B.
        DO 19 I=1,M
          DO 19 J=NP1,M
            VH(I,J)=VH(I,J)-2.D0*H1(I)*HU(J)
            C(I,J)=C(I,J)-2.D0*H2(I)*HU(J)
 19     CONTINUE    
C Overwrite annihlated Nth row.
        DO 20 J=1,M
          C(N,J)=V(J)
 20     CONTINUE     
 12   CONTINUE
C Convert upper diagonal matrix B to a real, positive matrix.
      N=1
      CONST=(1.D0,0.D0)
 21   CONTINUE
      NP1=N+1
C Find magnitudes of current row elements.
      EMAG=DREAL(C(N,N)*DCONJG(C(N,N)))
      EMAG=DSQRT(EMAG)
C Update unitary matrix U.
      CONST=DCONJG(CONST)*DCONJG(C(N,N))/EMAG
C Scale current row.
      DO 22 I=1,M
        U(N,I)=CONST*U(N,I)
 22   CONTINUE
C Replace diagonal element.  
      C(N,N)=DCMPLX(EMAG,0.D0)
      IF(N.EQ.M) GO TO 23
      EMAG=DREAL(C(N,NP1)*DCONJG(C(N,NP1)))
      EMAG=DSQRT(EMAG)
C Update unitary matrix VH.
      CONST=DCONJG(CONST)*DCONJG(C(N,NP1))/EMAG
C Scale next column.
      DO 24 I=1,M
         VH(I,NP1)=CONST*VH(I,NP1)
 24   CONTINUE
C Replace above diagonal element.   
      C(N,NP1)=DCMPLX(EMAG,0.D0)
      N=N+1
      GO TO 21
 23   CONTINUE
      RETURN
      END
C
      SUBROUTINE HHOLDER(V,U,N,M,MMAX)
C
C The subroutine HHOLDER annihilates the components of the vector V(M)
C beyond the Nth component for N<M. The Householder vector U is calculated
C and the annihilated vector W replaces V on return.
C
      DOUBLE COMPLEX U(MMAX),V(MMAX),W(MMAX)
      DOUBLE PRECISION ETA,AMAG,VWMAG,VR,VI
C No modification for N=M.
      IF(N.EQ.M) GO TO 10
C Find value of ETA.
      ETA=0.D0
      DO 11 I=N,M
        ETA=ETA+DREAL(V(I)*DCONJG(V(I)))
 11   CONTINUE
      ETA=DSQRT(ETA)
C Find magnitude and real and imaginary parts of V(N).
      AMAG=CDABS(V(N))
      VR=DREAL(V(N))
      VI=DIMAG(V(N))
C Find magnitude of V-W.
      VWMAG=2.D0*ETA*(ETA+AMAG)
      VWMAG=DSQRT(VWMAG)
C Construct vector U.
C Set initial values of components of U to zero.
      DO 12 I=1,M
        U(I)=(0.D0,0.D0)
 12   CONTINUE
C Set value of U(N).
      U(N)=V(N)+ETA*DCMPLX(VR/AMAG,VI/AMAG)
C Set values of remaining components of U.
      NP1=N+1
      DO 13 I=NP1,M
        U(I)=V(I)
 13   CONTINUE
C Normalize U to unit vector.
      DO 14 I=1,M
        U(I)=U(I)/VWMAG
 14   CONTINUE
C Initialize vector W as vector V.
      DO 15 I=1,M
        W(I)=V(I)
 15   CONTINUE  
C Construct vector W.
C Set value of W(N).
      W(N)=W(N)-VWMAG*U(N)
C Zero remaining components of W.
      DO 16 I=NP1,M
        W(I)=(0.D0,0.D0)
 16   CONTINUE
C Replace V by W.
      DO 17 I=1,M
        V(I)=W(I)
 17   CONTINUE  
 10   CONTINUE
      RETURN
      END
